`timescale 1ns / 1ps
//*****************************************************************************
// File Name            : spi_ap_test.v
//-----------------------------------------------------------------------------
// Function             : spi sample test bench
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2010/10/27
//*****************************************************************************/

module spi_ap_test;
        // Inputs
        reg clk;
        reg rstb;
        reg ss_d;
        reg sclk_d;
        reg [7:0] mosi_data;
        reg [2:0] spi_cmd;
        reg [23:0]spi_cnt;
        reg [31:0]miso_data;
        wire      miso;
        reg       rxd;
   

        // Outputs
        wire led_0;
        wire sclk;
        wire ss;
        wire ss0;
        wire ss1;
        wire ss2;
        wire ss3;
        wire      mosi;
        wire      txd;
   
   
        // Instantiate the Unit Under Test (UUT)
        spi_ap uut (
                .clk(clk), 
                .rstb(rstb), 
                .led_0(led_0), 
                .sclk(sclk), 
                .ss0(ss0), 
                .ss1(ss1), 
                .ss2(ss2), 
                .ss3(ss3), 
                .mosi(mosi), 
                .miso(miso),
                .rxd(rxd),
                .txd(txd)
        );

        initial begin
                // Initialize Inputs
                //clk = 0;
                rstb = 0;
                rxd = 1'b1;
                // Wait 100 ns for global reset to finish
                #100;
                rstb = 1;
                // Add stimulus here

        end


        initial begin
                // Initialize Inputs
                clk = 0;
                repeat(500000000) begin
                #12.5;
                clk = 1;
                #12.5;
                clk = 0;
                end
                $stop;
        end

assign ss = ss0;
      
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0) begin
    sclk_d <= 1'b0;
    ss_d <= 1'b0;
    end
  else
    begin
      sclk_d <= sclk;
      ss_d <= ss;
    end


always @ (posedge clk or negedge rstb )
  if (rstb==1'b0) begin
    mosi_data <= 8'd0;
    end
  else
    if ((sclk==1'b1)&&(sclk_d==1'b0))
      begin
        mosi_data <= {mosi_data[6:0],mosi};
      end
    else
      mosi_data <= mosi_data;
   
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    spi_cnt <= 24'd0;
  else
    if (ss==1'b0)
      spi_cnt <= 16'd0;
    else if ((sclk==1'b1)&&(sclk_d==1'b0))
      spi_cnt <= spi_cnt + 24'd1;
    else
      spi_cnt <= spi_cnt;
     
    
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    spi_cmd <= 3'b000;
  else
    if ((sclk==1'b1)&&(sclk_d==1'b0)&&(spi_cnt==24'd3))
      if (spi_cmd==3'b101)
        spi_cmd <= 3'b111;
      else
        spi_cmd <= mosi_data[2:0];
    else if ((ss==1'b0)&&(ss_d==1'b1))
      if (spi_cmd==3'b101)
        spi_cmd <= 3'b101;
      else
        spi_cmd <= 3'b000;
    else
      spi_cmd <= spi_cmd;
   

    
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
     miso_data <= {13'h0000,8'ha5,11'h00};
  else
    if (ss==1'b0)
      miso_data <= {13'h0000,8'ha5,11'h00};
    else if((sclk==1'b1)&&(sclk_d==1'b0)&&(spi_cmd ==3'b111))
       miso_data <= {miso_data[30:0],1'b0};
    else
      miso_data <= miso_data;

assign miso = miso_data[31];
   
    
endmodule

