//*****************************************************************************
// File Name            : spi_at93c86.v
//-----------------------------------------------------------------------------
// Function             : AT93C86 controller
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2010/10/27
//*****************************************************************************
module spi_at93c86 ( 
 clk, rstb,led_0,
  dev_sel,
  start,
  clk_end,
  wr_data,
  rd_data,rd_data_en,
  tx_fifo_data,tx_fifo_data_en
                     
);
  input clk;
  input rstb;
  output led_0;
  //spi_m_if
  output [1:0] dev_sel;
  output start;
  output [4:0]clk_end;
  output [31:0] wr_data;
  input [31:0] rd_data;
  input        rd_data_en; 
  //fifo
  output  [7:0] tx_fifo_data;    //tx fifo data 8bit
  output        tx_fifo_data_en; //tx fifo data enable 
   
  reg  [25:0] main_cnt;

  reg [1:0]dev_sel;
  reg start;
  reg [4:0]clk_end;
  reg we_doen;
  reg [10:0]adr;
  reg [7:0] data;
  reg [31:0]wr_data; 
  wire [31:0]rd_data; 
  wire rd_data_en;
  //wire rd_fifo_data_en;
  reg read_mode;
 
 
  reg  [7:0] tx_fifo_data;    //tx fifo data 8bit
  reg        tx_fifo_data_en; //tx fifo data enable 

  reg        rd;
  reg [3:0] msg_cnt;  //tx fifo data 8bit
  reg [1:0] msg_type;
  wire [31:0]msg_data;
   
function [7:0] hex2ascii;
  input [3:0] hex_data;
  begin
    if (hex_data<4'ha)
      hex2ascii = 8'h30 + hex_data;
    else
      hex2ascii = 8'h37 + hex_data;
  end
endfunction
   
   
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0) begin 
    main_cnt <= 26'b0;
  end
  //else if (main_cnt == 26'd7999999) begin
  else if (main_cnt == 26'd39999999) begin
    main_cnt <= 26'b0;
    end
  else begin
    main_cnt <= main_cnt + 26'd1;
  end

assign led_0 = main_cnt[25];


// for AT93C86 (EEPROM)
  
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0) begin
    dev_sel <= 2'b00; 
    start <= 1'b0;
    wr_data <= 8'h00;
    we_doen <= 1'b0;
    clk_end <= 5'd0;
    end
  else
    if ((we_doen==1'b0)&&(main_cnt ==26'd100000))//write eable
      begin
        dev_sel <= 2'b00; 
        start <= 1'b1;
        we_doen <= 1'b1;
        clk_end <= 5'd30;
        wr_data <= 32'h98000000;
      end
    else if ((main_cnt ==26'd500000)||//write
             (main_cnt ==26'd600000))  
      begin
        dev_sel <= 2'b00; 
        start <= 1'b1;
        clk_end <= 5'd22;
        wr_data <= {3'b101,adr,data,10'h000} ;
        //wr_data <= {3'b101,adr,8'hc5,10'h000} ;
        we_doen <= we_doen ;
      end
    else if ((main_cnt==26'd690000)||//read
             (main_cnt==26'd800000))
      begin
        dev_sel <= 2'b00; 
        start <= 1'b1;
        clk_end <= 5'd22;
        //wr_data <= {3'b110,adr,data,10'h000} ;
        wr_data <= {3'b110,adr,18'h00000} ;
        we_doen <= we_doen ;
      end
    else
      begin
        dev_sel <= 2'b00; 
        start <= 1'b0;
        wr_data <= wr_data;
        we_doen <= we_doen ;
      end

always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    begin
      adr <= 11'b10000000001; 
      data <= 8'h00; 
    end   
  else   
    if ((main_cnt==26'd550000)||
        (main_cnt==26'd640000))
      begin
        adr <= adr + 11'h1;
        data <= data - 8'h01;
      end
    else if (main_cnt==26'd645000)
      begin
        adr <= adr - 11'h2;
        data <= data ;
      end
    else if ((main_cnt==26'd780000)||
             (main_cnt==26'd880000))
      begin
        adr <= adr + 11'h1;
        data <= data ;
      end
    else
      begin
        adr <= adr;
        data <= data ;
      end


   
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    begin
      rd <= 1'b0; 
    end   
  else   
    if ((read_mode==1'b1)&&
        (rd_data_en==1'b1))
      begin
        rd <= 1'b1;
      end
   else
      begin   
        rd <= 1'b0; 
      end // else: !if(main_cnt ==26'd3500000)
   
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    read_mode <= 1'b0;
  else if (main_cnt ==26'd690000)
    read_mode <= 1'b1;
  else if (main_cnt ==26'd1000000)
    read_mode <= 1'b0;
  else
    read_mode <= read_mode;
   

//re232c message

always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    msg_type <= 2'b0;
  else
    if (start==1'b1)
      if(read_mode==1'b0)
        msg_type <= 2'b00;
      else 
        msg_type <= 2'b01;
    else
      msg_type <= msg_type;
   
assign msg_data =(msg_type == 2'b00)? wr_data[17:10]:
                 (msg_type == 2'b01)? rd_data[7:0]:32'h00000000;
   
   
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    msg_cnt <= 4'd0;
  else
    if (((start==1'b1)&&(read_mode==1'b0))||(rd==1'b1))
      msg_cnt <= 4'd1;
    else
      if (msg_cnt==4'd0)
        msg_cnt <= 4'd0;
      else
        msg_cnt <= msg_cnt + 4'd1;
    
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    begin
      tx_fifo_data <= 8'h00;
      tx_fifo_data_en <= 1'b0;
    end
  else
    case(msg_cnt)
      4'd0:begin
        tx_fifo_data <= 8'h00;
        tx_fifo_data_en <= 1'b0;
        end
      4'd1:begin
        if (msg_type==2'b00) 
          begin
            tx_fifo_data <= 8'h57; //"W"
            tx_fifo_data_en <= 1'b1;
          end
        else if (msg_type==2'b01) 
          begin
            tx_fifo_data <= 8'h52; //"R"
            tx_fifo_data_en <= 1'b1;
          end
        else if (msg_type==2'b10) 
          begin
            tx_fifo_data <= 8'h52; //"R"
            tx_fifo_data_en <= 1'b1;
          end
        else  
          begin
            tx_fifo_data <= 8'h45; //"E"
            tx_fifo_data_en <= 1'b1;
          end
        end
      4'd2:begin
        tx_fifo_data <= 8'h5f; // "-"
        tx_fifo_data_en <= 1'b1;
        end
      4'd3:begin
             tx_fifo_data <= hex2ascii({1'b0,adr[10:8]});
             tx_fifo_data_en <= 1'b1;
          end
      4'd4:begin
            tx_fifo_data <= hex2ascii({adr[7:4]});
            tx_fifo_data_en <= 1'b1;
          end
      4'd5:begin
            tx_fifo_data <= hex2ascii({adr[3:0]});
            tx_fifo_data_en <= 1'b1;
          end
      4'd6:begin
        tx_fifo_data <= 8'h5f; // "-"
        tx_fifo_data_en <= 1'b1;
        end
      4'd7:begin
        tx_fifo_data <=  hex2ascii(msg_data[7:4]);
        tx_fifo_data_en <= 1'b1;
        end
      4'd8:begin
        tx_fifo_data <=  hex2ascii(msg_data[3:0]);
        tx_fifo_data_en <= 1'b1;
        end
      4'd9:begin
        tx_fifo_data <= 8'h0a; // LF
        tx_fifo_data_en <= 1'b1;
        end
      default
        begin
          tx_fifo_data <= 8'h00;
          tx_fifo_data_en <= 1'b0;
        end
    endcase
                 
     
endmodule