//*****************************************************************************
// File Name            : spi_m_if.v
//-----------------------------------------------------------------------------
// Function             : spi master if
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2010/10/27
//******************************************************************************
module spi_m_if ( 
  clk, rstb,
  sclk,ss0,ss1,ss2,ss3,mosi,miso,     
  //we,wr,rd,adr,wr_data,rd_data,busy
  dev_sel,start,busy,
  clk_end,
  wr_data,
  rd_data,rd_data_en      
);
  input clk;  //clk 40MHz
  input rstb;
  //SPI
  output sclk;
  output ss0;
  output ss1;
  output ss2;
  output ss3;
  output mosi;
  input miso;
  //interface
  /* 
  input we;
  input wr;
  input rd;
  input [10:0]adr;
  input [7:0]wr_data;
  output [7:0]rd_data;
  output     busy;
  */
  input [1:0]dev_sel; //foCXI
  input start;        //ʐMJn
  output busy;        //ʐM
  input [4:0]clk_end; //ʐMNbN
  input [31:0] wr_data; //݃f[^
  output [31:0] rd_data;//ǂݏof[^
  output        rd_data_en; //ǂݏof[^Cl[u
 
  reg   sclk;
  reg   mosi;
  reg   start_d1;
  wire  start_sig;
  reg [4:0]clk_end_cnt;
  reg      busy;
  reg [31:0]mosi_data;
  reg [31:0]miso_data;
  reg [31:0]rd_data;
  reg [11:0] cnt_1clk;
  reg [5:0] clk_cnt;
  reg [15:0] state_cnt;
  reg        rd_data_en;
 
  parameter p_1bit_cnt = 12'd100;
  parameter p_mosi_chg = 12'd10;
  parameter p_miso_trg = p_1bit_cnt - 12'd1;
   
// wr,rd 1clk delay   
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0) begin 
     start_d1 <= 1'b0;
  end
  else begin
     start_d1 <= start;
  end

// strat signal
assign start_sig= ((start==1'b1)&&(start_d1==1'b0))? 1'b1:1'b0;

//end_cnt hold
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    clk_end_cnt <= 5'd0;
  else
    if (start_sig==1'b1)
      clk_end_cnt <= clk_end - 5'd1;
    else
      clk_end_cnt <= clk_end_cnt ;
   
//busy signel
   
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0) 
     busy <= 1'b0;
  else
    if (start_sig==1'b1)
      busy <= 1'b1;
    else if ((cnt_1clk == p_1bit_cnt)&&(clk_cnt==clk_end_cnt))
      busy <= 1'b0;
    else
      busy <= busy;

//SCLK generator
   
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0) begin 
    cnt_1clk <= 12'd0;
    clk_cnt <= 5'd0;
    end
  else 
    if (busy==1'b0) begin
      cnt_1clk <= 12'd0;
      clk_cnt <= 5'd0;
    end
    else if (cnt_1clk == p_1bit_cnt) begin
       cnt_1clk <= 12'd0;
       if (clk_cnt==clk_end_cnt)
         clk_cnt <= 5'd0;
       else
         clk_cnt <= clk_cnt + 5'd1;
       end
    else begin
      cnt_1clk <= cnt_1clk + 12'd1;
      clk_cnt <= clk_cnt;
  end


always @ (posedge clk or negedge rstb )
  if (rstb==1'b0) 
    sclk <= 1'b0;
  else 
    if (busy==1'b0) 
      sclk <= 1'b0;
    else if (cnt_1clk == p_1bit_cnt) 
      sclk <= 1'b0;
    else if (cnt_1clk == {1'b0,p_1bit_cnt[11:1]}) 
      sclk <= 1'b1;
    else
      sclk <= sclk;

//ss
   
assign ss0 =(dev_sel==2'd0)? busy:1'b0;
assign ss1 =(dev_sel==2'd1)? busy:1'b0;
assign ss2 =(dev_sel==2'd2)? busy:1'b0;
assign ss3 =(dev_sel==2'd3)? busy:1'b0;
   
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    mosi_data <= 32'b0;
  else
    if(start_sig==1'b1)
      mosi_data <=wr_data;
    else if (cnt_1clk==p_mosi_chg)
      mosi_data <= {mosi_data[30:0],1'b0};        
    else
      mosi_data <= mosi_data;


always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    mosi <= 1'b0;
  else
    if (busy==1'b1)
      if (cnt_1clk==p_mosi_chg)
        mosi <= mosi_data[31];
      else
        mosi <= mosi;
    else
    mosi <= 1'b0;

always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
     miso_data <= 32'd0;
  else
    if (cnt_1clk==p_miso_trg)  
      miso_data <= {miso_data[30:0],miso};
    else
      miso_data <= miso_data;
                         
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
     rd_data <= 32'd0;
  else
    if ((cnt_1clk == p_1bit_cnt)&&(clk_cnt==clk_end_cnt))  
      rd_data <= miso_data;
    else
      rd_data <= rd_data;
                            
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
     rd_data_en <= 1'b0;
  else
    if ((cnt_1clk == p_1bit_cnt)&&(clk_cnt==clk_end_cnt))  
      rd_data_en <= 1'b1;
    else
      rd_data_en <= 1'b0;
   
endmodule






