//*****************************************************************************
// File Name            : spi_mcp3002_4922.v
//-----------------------------------------------------------------------------
// Function             : MCP3002,MCP4922 controller
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2010/10/27
//*****************************************************************************
module spi_mcp3002_4922 ( 
 clk, rstb,led_0,
  dev_sel,
  start,
  clk_end,
  wr_data,
  rd_data,rd_data_en,
  tx_fifo_data,tx_fifo_data_en
                     
);
  input clk;
  input rstb;
  output led_0;
  //spi_m_if
  output [1:0] dev_sel;
  output start;
  output [4:0]clk_end;
  output [31:0] wr_data;
  input [31:0] rd_data;
  input        rd_data_en; 
  //fifo
  output  [7:0] tx_fifo_data;    //tx fifo data 8bit
  output        tx_fifo_data_en; //tx fifo data enable 
   
  reg  [25:0] main_cnt;

  reg [1:0]dev_sel;
  reg start;
  reg [4:0]clk_end;
  reg [11:0] data;
  reg [31:0]wr_data; 
  wire [31:0]rd_data; 
  wire rd_data_en;
 
  reg  [7:0] tx_fifo_data;    //tx fifo data 8bit
  reg        tx_fifo_data_en; //tx fifo data enable 

  reg [3:0] msg_cnt;  //tx fifo data 8bit
   
function [7:0] hex2ascii;
  input [3:0] hex_data;
  begin
    if (hex_data<4'ha)
      hex2ascii = 8'h30 + hex_data;
    else
      hex2ascii = 8'h37 + hex_data;
  end
endfunction
   
   
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0) begin 
    main_cnt <= 26'b0;
  end
  //else if (main_cnt == 26'd7999999) begin
  //else if (main_cnt == 26'd39999999) begin
  //else if (main_cnt ==   26'd9999999) begin
  else if (main_cnt ==   26'd199999) begin
    main_cnt <= 26'b0;
    end
  else begin
    main_cnt <= main_cnt + 26'd1;
  end

assign led_0 = main_cnt[25];


// for MCP4922 (EEPROM)
  
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0) begin
    dev_sel <= 2'b01; 
    start <= 1'b0;
    wr_data <= 8'h00;
    clk_end <= 5'd0;
    end
  else
    if (main_cnt ==26'd20000)//write mcp4922
      begin
        start <= 1'b1;
        clk_end <= 5'd16;
        wr_data <= {4'b0111,data,16'h0000} ;//ch_a,buf=1,gab=1,shdnb=1,data
        dev_sel <= 2'b01; 
      end
    else  if (main_cnt ==26'd120000)//read mcp3002
      begin
        start <= 1'b1;
        clk_end <= 5'd15;
        wr_data <= {5'b1101,28'b00000000000} ;//start,SGI,ODD,MSBF
        dev_sel <= 2'b10; 
      end
    else
      begin
        start <= 1'b0;
        wr_data <= wr_data;
        clk_end <= clk_end;
        dev_sel <= dev_sel; 
      end

always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    begin
      data <= 12'h000; 
    end   
  else   
    if (main_cnt==26'd100000)
      begin
        data <= data + 12'h010;
      end
    else
      begin
        data <= data ;
      end
   
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    msg_cnt <= 4'd0;
  else
    if (((dev_sel==2'b01)&&(start==1'b1))||
        ((dev_sel==2'b10)&&(rd_data_en==1'b1)))
      msg_cnt <= 4'd1;
    else
      if (msg_cnt==4'd0)
        msg_cnt <= 4'd0;
      else
        msg_cnt <= msg_cnt + 4'd1;
    
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    begin
      tx_fifo_data <= 8'h00;
      tx_fifo_data_en <= 1'b0;
    end
  else
    case(msg_cnt)
      4'd0:begin
        tx_fifo_data <= 8'h00;
        tx_fifo_data_en <= 1'b0;
        end
      4'd1:begin
          if (dev_sel==2'b01) 
            begin
              tx_fifo_data <= 8'h44; //"D"
              tx_fifo_data_en <= 1'b1;
            end
          else
            begin
              tx_fifo_data <= 8'h41; //"A"
              tx_fifo_data_en <= 1'b1;
            end
         
        end
      4'd2:begin
          if (dev_sel==2'b01) 
            begin
              tx_fifo_data <= 8'h41; //"A"
              tx_fifo_data_en <= 1'b1;
            end
          else
            begin
              tx_fifo_data <= 8'h44; //"D"
              tx_fifo_data_en <= 1'b1;
            end
         
        end
      4'd3:begin
        tx_fifo_data <= 8'h5f; // "-"
        tx_fifo_data_en <= 1'b1;
        end
      4'd4:begin
          if (dev_sel==2'b01) 
            begin
             tx_fifo_data <= hex2ascii(data[11:8]);
             tx_fifo_data_en <= 1'b1;
            end
          else
            begin
             tx_fifo_data <= hex2ascii(rd_data[9:6]);
             tx_fifo_data_en <= 1'b1;
            end
          end
      4'd5:begin
          if (dev_sel==2'b01) 
            begin
             tx_fifo_data <= hex2ascii(data[7:4]);
             tx_fifo_data_en <= 1'b1;
            end
          else
            begin
             tx_fifo_data <= hex2ascii(rd_data[5:2]);
             tx_fifo_data_en <= 1'b1;
            end
          end
      4'd6:begin
          if (dev_sel==2'b01) 
            begin
             tx_fifo_data <= hex2ascii(data[3:0]);
             tx_fifo_data_en <= 1'b1;
            end
          else
            begin
             tx_fifo_data <= hex2ascii({rd_data[1:0],2'b00});
             tx_fifo_data_en <= 1'b1;
            end
          end
      4'd7:begin
        tx_fifo_data <= 8'h0a; // LF
        tx_fifo_data_en <= 1'b1;
        end
      default
        begin
          tx_fifo_data <= 8'h00;
          tx_fifo_data_en <= 1'b0;
        end
    endcase
                 
     
endmodule